﻿#WORKUNIT('name', 'FDA_Drugs');
import std;
// Name:	Eric Bales
// Email: eric.bales@lnssi.com
//
// Data Source: http://www.fda.gov/Drugs/InformationOnDrugs/ucm079750.htm
//
// Instructions:
// 1. Download zip file containing the data from the URL above.
//		The layout of the data is also documented at this same URL.
// 2. Unzip the contents of the file (9 delimited-text files) to a local folder.
//		Examine the contents of the files to see what's in there, and how it's laid out.
// 3. Transfer the txt files to the Landing Zone.
// 4. Set the IP address for the Landing Zone here:
LandingZone_IP := '10.220.5.57';
// 5. Set the folder containing the data files on the Landing Zone here:  (Should end in / )
BaseDataDirectory := '/ftp/in/FDA_Drugs/';
// 6. Execute this BWR.  It will read the data from the Landing Zone and perform a simple
//    lookup within that data.
//

// ----------------------------------------------------------------------------------------
// Data declarations for the 9 data files provided by the FDA:

// Application Documents
//  Document addresses or URLs to letters, labels, reviews, Consumer Information Sheets,
//  FDA Talk Papers, and other types.
L_AppDoc := RECORD
	INTEGER4		AppDocID;
	STRING6			ApplNo;
	STRING4			SeqNo;
	STRING50		DocType;
	STRING100		DocTitle;
	STRING200		DocURL;
	STRING19		DocDate;
	STRING10		ActionType;
	INTEGER4		DuplicateCounter;
END;

// Application Document Type Lookup
//  Type of document that is linked, which relates to the AppDoc record.
L_AppDocType_Lookup := RECORD
	STRING50		AppDocType;
	INTEGER4		SortOrder;
END;

// Application
//  Application number and sponsor name.
L_Application := RECORD
	STRING6		ApplNo;
	STRING5		ApplType;
	STRING50	SponsorApplicant;
	BOOLEAN		MostRecentLabelAvailableFlag;
	BOOLEAN		CurrentPatentFlag;
	STRING10	ActionType;
	STRING3		ChemicalType;
	STRING2		TherapeuticPotential;
	STRING1		OrphanCode;
END;

// Document Type Lookup
//  Supplement type code and description to the application number.
L_DocType_Lookup := RECORD
	STRING4		DocType;
	STRING50	DocTypeDesc;
END;

// Product
//  This table contains the products included in each application. Includes form, dosage,
//  and route.
L_Product := RECORD
	STRING6		ApplNo;
	STRING3		ProductNo;
	STRING255	Form;
	STRING240	Dosage;
	UNSIGNED1	ProductMktStatus;  // (1=prescription, 2=OTC, 3=discontinued, 4=tentative approval)
	STRING100	TECode;
	BOOLEAN		ReferenceDrug;
	STRING125	DrugName;
	STRING255	ActiveIngred;
END;

// Product TECode
//  Therapeutic Equivalence Code for products.
L_Product_TECode := RECORD
	STRING6		ApplNo;
	STRING3		ProductNo;
	STRING50	TECode;
	INTEGER4	TESequence;
	UNSIGNED1	ProductMktStatus;
END;

// Supplements (Regulatory Action Date)
//  Approval history for each application. Includes supplement number and dates of approval.
L_RegActionDate := RECORD
	STRING6		ApplNo;
	STRING10	ActionType;
	STRING4		InDocTypeSeqNo;
	INTEGER4	DuplicateCounter;
	STRING19	ActionDate;
	STRING4		DocType;
END;

// Chemical Type Lookup
L_ChemicalType_Lookup := RECORD
	INTEGER4	ChemicalTypeID;
	STRING3		ChemicalTypeCode;
	STRING200	ChemicalTypeDescription;
END;

// Review Class Lookup
L_ReviewClass_Lookup := RECORD
	INTEGER4	ReviewClassID;
	STRING1		ReviewCode;
	STRING100	LongDescription;
	STRING100	ShortDescription;
END;

// ----------------------------------------------------------------------------------------
// Read the data from the Landing Zone.

// Filenames for the raw files on the landing zone, including path.
Raw_AppDoc := 							BaseDataDirectory+'AppDoc.txt';
Raw_AppDocType_Lookup :=		BaseDataDirectory+'AppDocType_Lookup.txt';
Raw_Application := 					BaseDataDirectory+'Application.txt';
Raw_DocType_Lookup :=				BaseDataDirectory+'DocType_lookup.txt';
Raw_Product :=							BaseDataDirectory+'Product.txt';
Raw_Product_TECode :=				BaseDataDirectory+'Product_tecode.txt';
Raw_RegActionDate :=				BaseDataDirectory+'RegActionDate.txt';
Raw_ChemicalType_Lookup :=	BaseDataDirectory+'ChemTypeLookup.txt';
Raw_ReviewClass_Lookup :=		BaseDataDirectory+'ReviewClass_Lookup.txt';

// For all data files, the input data is in text delimited format with tab as the
// delimiter, 1 heading row, and no quotes.

// Ingest and distribute AppDoc
dsAppDoc_In := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_AppDoc),
		L_AppDoc,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );
dsAppDoc := DISTRIBUTE(dsAppDoc_In, AppDocID);


// Ingest and distribute AppDocType_Lookup
dsAppDocType_Lookup := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_AppDocType_Lookup),
		L_AppDocType_Lookup,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );


// Ingest and distribute Application
//  The Boolean values are stored as strings in the raw data - have to convert them
L_Application_Raw := RECORD
		STRING6		ApplNo;
		STRING5		ApplType;
		STRING50	SponsorApplicant;
		STRING5		MostRecentLabelAvailableFlag; // Boolean
		STRING5		CurrentPatentFlag;						// Boolean
		STRING10	ActionType;
		STRING3		ChemicalType;
		STRING2		TherapeuticPotential;
		STRING1		OrphanCode;
END;
dsApplication_In := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_Application),
		L_Application_Raw,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );
dsApplication_Convert :=
	PROJECT(dsApplication_In,
					TRANSFORM(L_Application,
										SELF.MostRecentLabelAvailableFlag := IF(LEFT.MostRecentLabelAvailableFlag='False',False,True);
										SELF.CurrentPatentFlag := IF(LEFT.CurrentPatentFlag='False',False,True);
										SELF := LEFT;));
dsApplication := DISTRIBUTE(dsApplication_Convert, HASH(ApplNo));


// Ingest and distribute DocType_Lookup
dsDocType_Lookup := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_DocType_Lookup),
		L_DocType_Lookup,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );


// Ingest and distribute Product
//  The Boolean values are stored as strings in the raw data - have to convert them
L_Product_Raw := RECORD
		STRING6		ApplNo;
		STRING3		ProductNo;
		STRING255	Form;
		STRING240	Dosage;
		UNSIGNED1	ProductMktStatus;
		STRING100	TECode;
		STRING5		ReferenceDrug;  // Boolean
		STRING125	DrugName;
		STRING255	ActiveIngred;
END;
dsProduct_In := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_Product),
		L_Product_Raw,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );
dsProduct_Convert :=
	PROJECT(dsProduct_In,
					TRANSFORM(L_Product,
										SELF.ReferenceDrug := IF(LEFT.ReferenceDrug='False',False,True);
										SELF := LEFT;));
dsProduct := DISTRIBUTE(dsProduct_Convert, HASH(ApplNo));


// Ingest and distribute Product_TECode
dsProduct_TECode_In := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_Product_TECode),
		L_Product_TECode,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );
dsProduct_TECode := DISTRIBUTE(dsProduct_TECode_In, HASH(ApplNo));


// Ingest and distribute RegActionDate
dsRegActionDate_In := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_RegActionDate),
		L_RegActionDate,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );
dsRegActionDate := DISTRIBUTE(dsRegActionDate_In, HASH(ApplNo));


// Ingest and distribute ChemicalType_Lookup
dsChemicalType_Lookup := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_ChemicalType_Lookup),
		L_ChemicalType_Lookup,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );


// Ingest and distribute ReviewClass_Lookup
dsReviewClass_Lookup := DATASET(std.File.ExternalLogicalFilename(LandingZone_IP,Raw_ReviewClass_Lookup),
		L_ReviewClass_Lookup,
		CSV(HEADING(1), SEPARATOR('\t'), TERMINATOR(['\r\n','\n\r','\n','\r']), QUOTE(''), ASCII) );


// ----------------------------------------------------------------------------------------
// Use the data to do a simple lookup and output the results.

// Change this value to locate a different active ingredient.
ActiveIngredient := std.Str.ToUpperCase('Loratadine');


// Find all products with the active ingredient.
dsProducts := dsProduct(std.Str.Find(ActiveIngred,ActiveIngredient,1)>0);

// Put together all of the data for the found products and present in a single flattened layout.
L_Results := RECORD
		// From L_Product
		STRING6		ApplNo;
		STRING3		ProductNo;
		STRING255	Form;
		STRING240	Dosage;
		UNSIGNED1	ProductMktStatus;  // (1=prescription, 2=OTC, 3=discontinued, 4=tentative approval)
		STRING100	TECode;
		BOOLEAN		ReferenceDrug;
		STRING125	DrugName;
		STRING255	ActiveIngred;
		// From L_Application (ApplNo is the key)
		STRING5		ApplType;
		STRING50	SponsorApplicant;
		BOOLEAN		MostRecentLabelAvailableFlag;
		BOOLEAN		CurrentPatentFlag;
		STRING10	ActionType;
		STRING3		ChemicalType;
		STRING2		TherapeuticPotential;
		STRING1		OrphanCode;
		// From L_ChemicalType_Lookup (map ChemicalType to ChemicalTypeCode)
		INTEGER4	ChemicalTypeID							:= 0;
		STRING200	ChemicalTypeDescription			:= '';
		// From L_ReviewClass_Lookup (map TherapeuticPotential to ReviewCode)
		INTEGER4	ReviewClassID								:= 0;
		STRING100	LongDescription							:= '';
		STRING100	ShortDescription						:= '';
		// From L_RegActionDate (ApplNo is the key)
		STRING4		InDocTypeSeqNo							:= '';
		INTEGER4	DuplicateCounter						:= 0;
		STRING19	ActionDate									:= '';
		STRING4		DocType											:= '';
		// From L_DocType_Lookup (map DocType)
		STRING50	DocTypeDesc									:= '';
		// From L_AppDoc (ApplNo is the key)
		INTEGER4		AppDocID									:= 0;
		STRING4			SeqNo											:= '';
		STRING50		AppDocType								:= '';
		STRING100		DocTitle									:= '';
		STRING200		DocURL										:= '';
		STRING19		DocDate										:= '';
		// From L_AppDocType_Lookup (map AppDocType)
		INTEGER4		SortOrder									:= 0;
END;

// Convert to L_Results layout, and join Product and Application data.
//   This data is already distributed on ApplNo, so can do a LOCAL join.
dsProductApplication :=
	JOIN(dsProducts, dsApplication,
			 LEFT.ApplNo=RIGHT.ApplNo,
			 TRANSFORM(L_Results, SELF := LEFT; SELF := RIGHT;),
			 LOCAL);

// Join in the RegActionDate and AppDoc data
dsProdAppRegActionDate :=
	JOIN(dsProductApplication, dsRegActionDate,
			 LEFT.ApplNo=RIGHT.ApplNo,
			 TRANSFORM(L_Results, SELF := LEFT; SELF := RIGHT;),
			 LOCAL);
dsProdAppRegAppDoc :=
	JOIN(dsProdAppRegActionDate, dsAppDoc,
			 LEFT.ApplNo=RIGHT.ApplNo,
			 TRANSFORM(L_Results,
								 SELF.AppDocType := RIGHT.DocType;
								 SELF := LEFT; SELF := RIGHT;),
			 LOCAL);

// Dedup
dsDeduppedResults := DEDUP(SORT(dsProdAppRegAppDoc, RECORD), RECORD);

// Join in the lookups of the various explanatory code values.
//  Use LEFT OUTER join because these values are sometimes optional.
dsChemTypeLookup :=
	JOIN(dsDeduppedResults, dsChemicalType_Lookup,
			 LEFT.ChemicalType=RIGHT.ChemicalTypeCode,
			 TRANSFORM(L_Results, SELF := LEFT; SELF := RIGHT;),
			 LEFT OUTER, LOOKUP);
dsReviewClassLookup :=
	JOIN(dsChemTypeLookup, dsReviewClass_Lookup,
			 LEFT.TherapeuticPotential=RIGHT.ReviewCode,
			 TRANSFORM(L_Results, SELF := LEFT; SELF := RIGHT;),
			 LEFT OUTER, LOOKUP);
dsDocTypeLookup :=
	JOIN(dsReviewClassLookup, dsDocType_Lookup,
			 LEFT.DocType=RIGHT.DocType,
			 TRANSFORM(L_Results, SELF := LEFT; SELF := RIGHT;),
			 LEFT OUTER, LOOKUP);
dsAppDocTypeLookup :=
	JOIN(dsDocTypeLookup, dsAppDocType_Lookup,
			 LEFT.AppDocType=RIGHT.AppDocType,
			 TRANSFORM(L_Results, SELF := LEFT; SELF := RIGHT;),
			 LEFT OUTER, LOOKUP);

// Sort the results
dsResults := SORT(dsAppDocTypeLookup, ApplNo,ProductMktStatus,InDocTypeSeqNo,SeqNo,SortOrder,ActionType,ActionDate);

OUTPUT(dsResults, NAMED('Drug_Info_For_'+ActiveIngredient));
